@@ -1,4 +1,4 @@
-/* $Id: colorprocs.h,v 1.4 2006/12/07 22:49:35 erg Exp $ $Revision: 1.4 $ */
+/* $Id: colorprocs.h,v 1.5 2008/01/03 23:24:28 ellson Exp $ $Revision: 1.5 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -17,7 +17,7 @@
 #ifndef GV_COLORPROCS_H
 #define GV_COLORPROCS_H
 
-#include <color.h>
+#include "color.h"
 
 #ifdef __cplusplus
 extern "C" {
