@@ -1,4 +1,4 @@
-/* $Id: exeval.c,v 1.3 2005/10/26 01:32:10 erg Exp $ $Revision: 1.3 $ */
+/* $Id: exeval.c,v 1.5 2007/07/17 21:49:24 erg Exp $ $Revision: 1.5 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -25,10 +25,20 @@
 #include "config.h"
 #endif
 
+#ifdef GVDLL
+#define _BLD_sfio 1
+#endif
+
 #include "exlib.h"
 #include "exop.h"
 #include <string.h>
 #include <time.h>
+#ifdef WIN32
+#include <stdlib.h>
+#define srand48 srand
+#define drand48 rand
+#endif
+
 
 static Extype_t eval(Expr_t *, Exnode_t *, void *);
 
