@@ -110,7 +110,8 @@ static Agraph_t		*G;
 static Agnode_t		*N;
 static Agedge_t		*E;
 static objstack_t	*SP;
-static Agraph_t		*Gstack[32];
+#define GSTACK_SIZE 64
+static Agraph_t         *Gstack[GSTACK_SIZE];
 static int			GSP;
 
 static void subgraph_warn (void)
@@ -123,6 +124,9 @@ static void subgraph_warn (void)
 
 static void push_subg(Agraph_t *g)
 {
+	if (GSP >= GSTACK_SIZE) {
+		agerr (AGERR, "Gstack overflow in graph parser\n"); exit(1);
+	}
 	G = Gstack[GSP++] = g;
 }
 
@@ -400,7 +404,7 @@ concat3 (char* s1, char* s2, char*s3)
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 304 "../../lib/graph/parser.y"
+#line 308 "../../lib/graph/parser.y"
 {
 			int					i;
 			char				*str;
@@ -408,7 +412,7 @@ typedef union YYSTYPE
 			struct Agnode_t		*n;
 }
 /* Line 187 of yacc.c.  */
-#line 412 "y.tab.c"
+#line 416 "y.tab.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -421,7 +425,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 425 "y.tab.c"
+#line 429 "y.tab.c"
 
 #ifdef short
 # undef short
@@ -726,13 +730,13 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   322,   322,   321,   325,   332,   335,   335,   338,   340,
-     342,   344,   348,   350,   352,   356,   357,   360,   361,   364,
-     367,   368,   371,   374,   378,   379,   383,   384,   387,   388,
-     391,   392,   393,   396,   397,   398,   399,   402,   404,   408,
-     418,   421,   422,   423,   427,   426,   433,   435,   432,   440,
-     442,   439,   448,   450,   449,   452,   455,   454,   460,   461,
-     461,   462,   462,   463,   466,   476,   477,   480,   481
+       0,   326,   326,   325,   329,   336,   339,   339,   342,   344,
+     346,   348,   352,   354,   356,   360,   361,   364,   365,   368,
+     371,   372,   375,   378,   382,   383,   387,   388,   391,   392,
+     395,   396,   397,   400,   401,   402,   403,   406,   408,   412,
+     422,   425,   426,   427,   431,   430,   437,   439,   436,   444,
+     446,   443,   452,   454,   453,   456,   459,   458,   464,   465,
+     465,   466,   466,   467,   470,   480,   481,   484,   485
 };
 #endif
 
@@ -1696,17 +1700,17 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 322 "../../lib/graph/parser.y"
+#line 326 "../../lib/graph/parser.y"
     {begin_graph((yyvsp[(2) - (2)].str)); agstrfree((yyvsp[(2) - (2)].str));}
     break;
 
   case 3:
-#line 324 "../../lib/graph/parser.y"
+#line 328 "../../lib/graph/parser.y"
     {AG.accepting_state = TRUE; end_graph();}
     break;
 
   case 4:
-#line 326 "../../lib/graph/parser.y"
+#line 330 "../../lib/graph/parser.y"
     {
 					if (AG.parsed_g)
 						agclose(AG.parsed_g);
@@ -1716,87 +1720,87 @@ yyreduce:
     break;
 
   case 5:
-#line 332 "../../lib/graph/parser.y"
+#line 336 "../../lib/graph/parser.y"
     {AG.parsed_g = NULL;}
     break;
 
   case 6:
-#line 335 "../../lib/graph/parser.y"
+#line 339 "../../lib/graph/parser.y"
     {(yyval.str)=(yyvsp[(1) - (1)].str);}
     break;
 
   case 7:
-#line 335 "../../lib/graph/parser.y"
+#line 339 "../../lib/graph/parser.y"
     {(yyval.str)=0;}
     break;
 
   case 8:
-#line 339 "../../lib/graph/parser.y"
+#line 343 "../../lib/graph/parser.y"
     {Agraph_type = AGRAPH; AG.edge_op = "--";}
     break;
 
   case 9:
-#line 341 "../../lib/graph/parser.y"
+#line 345 "../../lib/graph/parser.y"
     {Agraph_type = AGRAPHSTRICT; AG.edge_op = "--";}
     break;
 
   case 10:
-#line 343 "../../lib/graph/parser.y"
+#line 347 "../../lib/graph/parser.y"
     {Agraph_type = AGDIGRAPH; AG.edge_op = "->";}
     break;
 
   case 11:
-#line 345 "../../lib/graph/parser.y"
+#line 349 "../../lib/graph/parser.y"
     {Agraph_type = AGDIGRAPHSTRICT; AG.edge_op = "->";}
     break;
 
   case 12:
-#line 349 "../../lib/graph/parser.y"
+#line 353 "../../lib/graph/parser.y"
     {Current_class = TAG_GRAPH;}
     break;
 
   case 13:
-#line 351 "../../lib/graph/parser.y"
+#line 355 "../../lib/graph/parser.y"
     {Current_class = TAG_NODE; N = G->proto->n;}
     break;
 
   case 14:
-#line 353 "../../lib/graph/parser.y"
+#line 357 "../../lib/graph/parser.y"
     {Current_class = TAG_EDGE; E = G->proto->e;}
     break;
 
   case 23:
-#line 375 "../../lib/graph/parser.y"
+#line 379 "../../lib/graph/parser.y"
     {attr_set((yyvsp[(1) - (3)].str),(yyvsp[(3) - (3)].str)); agstrfree((yyvsp[(1) - (3)].str)); agstrfree((yyvsp[(3) - (3)].str));}
     break;
 
   case 25:
-#line 380 "../../lib/graph/parser.y"
+#line 384 "../../lib/graph/parser.y"
     {attr_set((yyvsp[(1) - (1)].str),"true"); agstrfree((yyvsp[(1) - (1)].str)); }
     break;
 
   case 32:
-#line 393 "../../lib/graph/parser.y"
+#line 397 "../../lib/graph/parser.y"
     {agerror("syntax error, statement skipped");}
     break;
 
   case 36:
-#line 399 "../../lib/graph/parser.y"
+#line 403 "../../lib/graph/parser.y"
     {}
     break;
 
   case 37:
-#line 403 "../../lib/graph/parser.y"
+#line 407 "../../lib/graph/parser.y"
     {Current_class = TAG_GRAPH; /* reset */}
     break;
 
   case 38:
-#line 405 "../../lib/graph/parser.y"
+#line 409 "../../lib/graph/parser.y"
     {Current_class = TAG_GRAPH;}
     break;
 
   case 39:
-#line 409 "../../lib/graph/parser.y"
+#line 413 "../../lib/graph/parser.y"
     {
 					objport_t		rv;
 					rv.obj = (yyvsp[(1) - (2)].n);
@@ -1807,114 +1811,114 @@ yyreduce:
     break;
 
   case 40:
-#line 418 "../../lib/graph/parser.y"
+#line 422 "../../lib/graph/parser.y"
     {(yyval.n) = bind_node((yyvsp[(1) - (1)].str)); agstrfree((yyvsp[(1) - (1)].str));}
     break;
 
   case 42:
-#line 422 "../../lib/graph/parser.y"
+#line 426 "../../lib/graph/parser.y"
     { Port=(yyvsp[(2) - (2)].str);}
     break;
 
   case 43:
-#line 423 "../../lib/graph/parser.y"
+#line 427 "../../lib/graph/parser.y"
     {Port=concat3((yyvsp[(2) - (4)].str),":",(yyvsp[(4) - (4)].str));agstrfree((yyvsp[(2) - (4)].str)); agstrfree((yyvsp[(4) - (4)].str));}
     break;
 
   case 44:
-#line 427 "../../lib/graph/parser.y"
+#line 431 "../../lib/graph/parser.y"
     {Current_class = TAG_NODE; N = (Agnode_t*)((yyvsp[(1) - (1)].obj).obj);}
     break;
 
   case 45:
-#line 429 "../../lib/graph/parser.y"
+#line 433 "../../lib/graph/parser.y"
     {agstrfree((yyvsp[(1) - (3)].obj).port);Current_class = TAG_GRAPH; /* reset */}
     break;
 
   case 46:
-#line 433 "../../lib/graph/parser.y"
+#line 437 "../../lib/graph/parser.y"
     {begin_edgestmt((yyvsp[(1) - (1)].obj));}
     break;
 
   case 47:
-#line 435 "../../lib/graph/parser.y"
+#line 439 "../../lib/graph/parser.y"
     { E = SP->subg->proto->e;
 				  Current_class = TAG_EDGE; }
     break;
 
   case 48:
-#line 438 "../../lib/graph/parser.y"
+#line 442 "../../lib/graph/parser.y"
     {end_edgestmt();}
     break;
 
   case 49:
-#line 440 "../../lib/graph/parser.y"
+#line 444 "../../lib/graph/parser.y"
     {begin_edgestmt((yyvsp[(1) - (1)].obj));}
     break;
 
   case 50:
-#line 442 "../../lib/graph/parser.y"
+#line 446 "../../lib/graph/parser.y"
     { E = SP->subg->proto->e;
 				  Current_class = TAG_EDGE; }
     break;
 
   case 51:
-#line 445 "../../lib/graph/parser.y"
+#line 449 "../../lib/graph/parser.y"
     {end_edgestmt();}
     break;
 
   case 52:
-#line 448 "../../lib/graph/parser.y"
+#line 452 "../../lib/graph/parser.y"
     {mid_edgestmt((yyvsp[(2) - (2)].obj));}
     break;
 
   case 53:
-#line 450 "../../lib/graph/parser.y"
+#line 454 "../../lib/graph/parser.y"
     {mid_edgestmt((yyvsp[(2) - (2)].obj));}
     break;
 
   case 55:
-#line 453 "../../lib/graph/parser.y"
+#line 457 "../../lib/graph/parser.y"
     {mid_edgestmt((yyvsp[(2) - (2)].obj));}
     break;
 
   case 56:
-#line 455 "../../lib/graph/parser.y"
+#line 459 "../../lib/graph/parser.y"
     {mid_edgestmt((yyvsp[(2) - (2)].obj));}
     break;
 
   case 58:
-#line 460 "../../lib/graph/parser.y"
+#line 464 "../../lib/graph/parser.y"
     {(yyval.obj) = pop_gobj();}
     break;
 
   case 59:
-#line 461 "../../lib/graph/parser.y"
+#line 465 "../../lib/graph/parser.y"
     { anonsubg(); }
     break;
 
   case 60:
-#line 461 "../../lib/graph/parser.y"
+#line 465 "../../lib/graph/parser.y"
     {(yyval.obj) = pop_gobj();}
     break;
 
   case 61:
-#line 462 "../../lib/graph/parser.y"
+#line 466 "../../lib/graph/parser.y"
     { anonsubg(); }
     break;
 
   case 62:
-#line 462 "../../lib/graph/parser.y"
+#line 466 "../../lib/graph/parser.y"
     {(yyval.obj) = pop_gobj();}
     break;
 
   case 63:
-#line 463 "../../lib/graph/parser.y"
+#line 467 "../../lib/graph/parser.y"
     {subgraph_warn(); (yyval.obj) = pop_gobj();}
     break;
 
   case 64:
-#line 467 "../../lib/graph/parser.y"
+#line 471 "../../lib/graph/parser.y"
     { Agraph_t	 *subg;
 				if ((subg = agfindsubg(AG.parsed_g,(yyvsp[(2) - (2)].str)))) aginsert(G,subg);
 				else subg = agsubg(G,(yyvsp[(2) - (2)].str)); 
@@ -1925,28 +1929,28 @@ yyreduce:
     break;
 
   case 65:
-#line 476 "../../lib/graph/parser.y"
-    {(yyval.str) = (yyvsp[(1) - (1)].str); }
-    break;
-
-  case 66:
-#line 477 "../../lib/graph/parser.y"
-    {(yyval.str) = (yyvsp[(1) - (1)].str); }
-    break;
-
-  case 67:
 #line 480 "../../lib/graph/parser.y"
     {(yyval.str) = (yyvsp[(1) - (1)].str); }
     break;
 
-  case 68:
+  case 66:
 #line 481 "../../lib/graph/parser.y"
+    {(yyval.str) = (yyvsp[(1) - (1)].str); }
+    break;
+
+  case 67:
+#line 484 "../../lib/graph/parser.y"
+    {(yyval.str) = (yyvsp[(1) - (1)].str); }
+    break;
+
+  case 68:
+#line 485 "../../lib/graph/parser.y"
     {(yyval.str) = concat((yyvsp[(1) - (3)].str),(yyvsp[(3) - (3)].str)); agstrfree((yyvsp[(1) - (3)].str)); agstrfree((yyvsp[(3) - (3)].str));}
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 1950 "y.tab.c"
+#line 1954 "y.tab.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2160,7 +2164,7 @@ yyreturn:
 }
 
 
-#line 483 "../../lib/graph/parser.y"
+#line 487 "../../lib/graph/parser.y"
 
 #ifdef UNUSED
 /* grammar allowing port variants */
