@@ -1,4 +1,4 @@
-/* $Id: gvplugin_loadimage.h,v 1.3 2007/10/03 19:19:29 ellson Exp $ $Revision: 1.3 $ */
+/* $Id: gvplugin_loadimage.h,v 1.5 2008/04/18 14:10:43 glenlow Exp $ $Revision: 1.5 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -28,6 +28,9 @@ extern void gvdevice_printf(GVJ_t * job, const char *format, ...);
 extern "C" {
 #endif
 
+extern boolean gvusershape_file_access(usershape_t *us);
+extern void gvusershape_file_release(usershape_t *us);
+
     struct gvloadimage_engine_s {
 	void (*loadimage) (GVJ_t *job, usershape_t *us, boxf b, boolean filled);
     };
