@@ -1,4 +1,4 @@
-/* $Id: circle.c,v 1.2 2006/01/17 18:21:43 erg Exp $ $Revision: 1.2 $ */
+/* $Id: circle.c,v 1.3 2008/03/03 23:01:52 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -212,7 +212,7 @@ static void setChildSubtreeSpans(Agraph_t * g, Agnode_t * n)
 
 static void setSubtreeSpans(Agraph_t * sg, Agnode_t * center)
 {
-    SPAN(center) = 2 * PI;
+    SPAN(center) = 2 * M_PI;
     setChildSubtreeSpans(sg, center);
 }
 
