@@ -1,4 +1,4 @@
-/* $Id: gvrender_quartz.c,v 1.4 2008/02/08 10:59:25 glenlow Exp $ $Revision: 1.4 $ */
+/* $Id: gvrender_quartz.c,v 1.8 2008/06/03 05:27:28 glenlow Exp $ $Revision: 1.8 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -25,54 +25,10 @@
 #include "gvplugin_render.h"
 #include "graph.h"
 
-#include <ApplicationServices/ApplicationServices.h>
+#include "gvplugin_quartz.h"
 
-typedef enum {
-	FORMAT_BMP,
-	FORMAT_EXR,
-	FORMAT_GIF,
-	FORMAT_JPEG,
-	FORMAT_JPEG2000,
-	FORMAT_PDF,
-	FORMAT_PICT,
-	FORMAT_PNG,
-	FORMAT_PSD,
-	FORMAT_SGI,
-	FORMAT_TIFF,
-	FORMAT_TGA
-} format_type;
-
-static const int BYTE_ALIGN = 15;			/* align to 16 bytes */
-static const int BITS_PER_COMPONENT = 8;	/* bits per color component */
-static const int BYTES_PER_PIXEL = 4;		/* bytes per pixel */
-
-/* Uniform Type Identifiers corresponding to each format_type */
-static CFStringRef format_uti [] = {
-	CFSTR("com.microsoft.bmp"),
-	CFSTR("com.ilm.openexr-image"),
-	CFSTR("com.compuserve.gif"),
-	CFSTR("public.jpeg"),
-	CFSTR("public.jpeg-2000"),
-	CFSTR("com.adobe.pdf"),
-	CFSTR("com.apple.pict"),
-	CFSTR("public.png"),
-	CFSTR("com.adobe.photoshop-image"),
-	CFSTR("com.sgi.sgi-image"),
-	CFSTR("public.tiff"),
-	CFSTR("com.truevision.tga-image")
-};
-
-extern size_t gvdevice_write(GVJ_t *job, const unsigned char *s, unsigned int len);
-
-static size_t device_data_consumer_put_bytes (void *info, const void *buffer, size_t count)
-{
-	return gvdevice_write((GVJ_t *)info, (const unsigned char*)buffer, count);
-}
-
-static CGDataConsumerCallbacks device_data_consumer_callbacks = {
-	device_data_consumer_put_bytes,
-	NULL
-};
+static CGFloat dashed[] = {6.0};
+static CGFloat dotted[] = {2.0, 6.0};
 
 static void quartzgen_begin_job(GVJ_t *job)
 {
@@ -103,6 +59,8 @@ static void quartzgen_end_job(GVJ_t *job)
 				CGImageDestinationFinalize(image_destination);
 				
 				/* clean up */
+				if (image_destination)
+					CFRelease(image_destination);
 				CGImageRelease(image);
 				CGDataConsumerRelease(data_consumer);
 			}
@@ -167,10 +125,13 @@ static void quartzgen_begin_page(GVJ_t *job)
 					job->height,												/* height in pixels */
 					BITS_PER_COMPONENT,											/* bits per component */
 					(job->width * BYTES_PER_PIXEL + BYTE_ALIGN) & ~BYTE_ALIGN,	/* bytes per row: align to 16 byte boundary */
-					color_space,												/* color space: SRGB */
+					color_space,												/* color space: sRGB */
 					kCGImageAlphaPremultipliedFirst								/* bitmap info: premul ARGB has best support in OS X */
 				);
 				job->imagedata = CGBitmapContextGetData((CGContextRef)job->context);
+				
+				/* clean up */
+				CGColorSpaceRelease(color_space);
 			}
 			break;
 		}
@@ -220,6 +181,8 @@ static void quartzgen_textpara(GVJ_t *job, pointf p, textpara_t *para)
 {
 	CGContextRef context = (CGContextRef)job->context;
 	
+	CFStringRef str = CFStringCreateWithBytesNoCopy(kCFAllocatorDefault, (const UInt8 *)para->str, strlen(para->str), kCFStringEncodingUTF8, FALSE, kCFAllocatorNull);
+	if (str) {
 	/* set up the Core Text line */
 	CFStringRef attribute_keys[] = {
 		kCTFontAttributeName,
@@ -237,24 +200,23 @@ static void quartzgen_textpara(GVJ_t *job, pointf p, textpara_t *para)
 		sizeof(attribute_values) / sizeof(attribute_values[0]),
 		&kCFTypeDictionaryKeyCallBacks,
 		&kCFTypeDictionaryValueCallBacks);
-	CFStringRef str = CFStringCreateWithBytesNoCopy(kCFAllocatorDefault, (const UInt8 *)para->str, strlen(para->str), kCFStringEncodingUTF8, FALSE, kCFAllocatorNull);
 	CFAttributedStringRef attributed_str = CFAttributedStringCreate(kCFAllocatorDefault, str, attributes);
 	CTLineRef line = CTLineCreateWithAttributedString(attributed_str);
 	
 	/* adjust text position */
 	switch (para->just) {
-    case 'r':
-        p.x -= para->width;
-        break;
-    case 'l':
-        p.x -= 0.0;
-        break;
-    case 'n':
-    default:
-        p.x -= para->width / 2.0;
-        break;
-    }
-    p.y += para->yoffset_centerline;
+		case 'r':
+			p.x -= para->width;
+			break;
+		case 'l':
+			p.x -= 0.0;
+			break;
+		case 'n':
+		default:
+			p.x -= para->width / 2.0;
+			break;
+		}
+		p.y += para->yoffset_centerline;
 	
 	/* draw it */
 	CGContextSetTextPosition(context, p.x, p.y);
@@ -269,6 +231,8 @@ static void quartzgen_textpara(GVJ_t *job, pointf p, textpara_t *para)
 	for (i = 0; i < sizeof(attribute_values) / sizeof(attribute_values[0]); ++i)
 		CFRelease(attribute_values[i]);
 	CFRelease(fontname);
+	}
+
 }
 
 static void quartzgen_path(GVJ_t *job, int filled)
@@ -279,6 +243,27 @@ static void quartzgen_path(GVJ_t *job, int filled)
 	if (filled)
 		CGContextSetRGBFillColor(context, job->obj->fillcolor.u.RGBA [0], job->obj->fillcolor.u.RGBA [1], job->obj->fillcolor.u.RGBA [2], job->obj->fillcolor.u.RGBA [3]);
 	CGContextSetRGBStrokeColor(context, job->obj->pencolor.u.RGBA [0], job->obj->pencolor.u.RGBA [1], job->obj->pencolor.u.RGBA [2], job->obj->pencolor.u.RGBA [3]);
+	
+	/* set up line style */
+	const CGFloat *segments;
+	size_t segment_count;
+	switch (job->obj->pen) {
+	case PEN_DASHED:
+		segments = dashed;
+		segment_count = sizeof(dashed)/sizeof(CGFloat);
+		break;
+	case PEN_DOTTED:
+		segments = dotted;
+		segment_count = sizeof(dotted)/sizeof(CGFloat);
+		break;
+	default:
+		segments = NULL;
+		segment_count = 0;
+		break;
+	}
+	CGContextSetLineDash(context, 0.0, segments, segment_count);
+
+	/* set up line width */
 	CGContextSetLineWidth(context, job->obj->penwidth); // *job->scale.x);
 	
 	/* draw the path */
